% ****************************************************************
% ************ Messtechnik-Labor Abtastung-Skript ****************
% ****************************************************************

% Institut f�r Automatisierungs- und Regelungstechnik (ACIN)
% Author: MBI
% Date: 21.10.2015
% Version: 1.02

cd 'C:\Users\MT-LAB\Desktop\Abtastung';

if exist('visa_obj', 'var')                                                         % Close visa object if exists
    fclose(visa_obj); 
end

%close all; clear all; clc;                                                      % Delete all data and close all windows

%% Open VISA
% visa_obj = openDAQ('USB0::0x0957::0x1718::TW56194510::0::INSTR');               % Connect to the device
% visa_obj = openDAQ('USB0::0x0957::0x1718::TW56194511::0::INSTR');               % Connect to the device
% visa_obj = openDAQ('USB0::0x0957::0x1718::TW56194503::0::INSTR');               % Connect to the device

%% init

Samplerate = 20*1000;                                                             % DAQ sample rate
Datapoints = 10000;                                                             % DAQ datapoints 3000

% Initialisierung Analog Inputs 101 bis 104.

initAI(visa_obj, 101, Samplerate, Datapoints, 'BIP', 10);
initAI(visa_obj, 102, Samplerate, Datapoints, 'BIP', 10);
initAI(visa_obj, 103, Samplerate, Datapoints, 'BIP', 10);
initAI(visa_obj, 104, Samplerate, Datapoints, 'BIP', 10);


fprintf(visa_obj, 'TRIG:SOUR EXTD');                                            % Select trigger source external digital trigger for inputs
fprintf(visa_obj, 'TRIG:DTRG:POL POS');                                         % Select trigger polarity positive for inputs
fprintf(visa_obj, 'TRIG:TYP POST');                                             % Select trigger type post for inputs


% Initialisierung Analog Outputs 201 bis 202.

initAO(visa_obj, 201, 'BIP', 500000, 0);         % 0 ... inifinite output-iterations
initAO(visa_obj, 202, 'BIP', 500000, 0);


fprintf(visa_obj, 'OUTP:TRIG:SOUR EXTD');                                       % Select trigger source external digital trigger for outputs
fprintf(visa_obj, 'OUTP:TRIG:DTRG:POL POS');                                    % Select trigger polarity positive for outputs
fprintf(visa_obj, 'OUTP:TRIG:TYP POST');                                        % Select trigger type post for outputs

fprintf(visa_obj, 'SYST:ERR?');                                                 % Query for errors
fscanf(visa_obj)

%% Aquire Waveform
clc;                                                                            % Clear output window
close all;                                                                      % Close all open windows

% prepare figures 
figure('units', 'normalized', 'outerposition', [0 0 1 1], 'name', 'Input and windowed input signal');                 % Open plot window
maximize();                                                                   % Show the window maximised
ah31 = plot(0, 0, 'LineWidth', 2);                                              % Create axis_handle 31
xlabel('Time (ms)');                                                            % Set xlabel
ylabel('Amplitude (V)');                                                        % Set ylabel
hold on;                                                                        % Hold on that a second line dos not delete the first
ah32 = plot(0, 0, 'r', 'LineWidth', 2);                                         % Create axis_handle 32
hold off;
grid on;                                                                        % Eenables the grid in the plot

set(findall(gcf(),'type','text'),'fontSize',20,'fontname','Tahoma')                % Replace all text properties in the plot
set(findall(gca(),'type','axes'),'fontSize',20,'fontname','Tahoma')                % Replace all axes text properties in the plot

figure('units', 'normalized', 'outerposition', [0 0 1 1], 'name', 'Timesignals and spektrum');                 % Open plot window
maximize();                                                                     % Show the window maximised

subplot(2,1,1);                                                                 % Create subplot 1
ah11 = plot(0, 0, 'LineWidth', 2);                                              % Create axis_handle 11
xlabel('Zeit (ms)');                                                            % Set xlabel
ylabel('Amplitude (V)');                                                        % Set ylabel
hold on;                                                                        % Hold on that a second line dos not delete the first
ah12 = plot(0, 0, 'r', 'LineWidth', 2);                                         % Create axis_handle 12
ylim([-6 6]);                                                                   % Set y_limit
hold off;
grid on;                                                                        % Enables the grid in the plot

set(findall(gcf(),'type','text'),'fontSize',20,'fontname','Tahoma')                % Replace all text properties in the plot
set(findall(gca(),'type','axes'),'fontSize',20,'fontname','Tahoma')                % Replace all axes text properties in the plot

subplot(2,1,2);                                                                 % Create subplot 2
ah21 = plot(0, 0, 'b', 'LineWidth', 2);                                         % Create axis_handle 21
hold on;
ah22 = plot(0, 0, 'r', 'LineWidth', 2);                                         % Create axis_handle 22
xlabel('Frequenz (kHz)');
ylabel('Amplitude (dBV)');
hold off;
ylim([-120 15]);
grid on;

set(findall(gcf(),'type','text'),'fontSize',20,'fontname','Tahoma')                % Replace all text properties in the plot
set(findall(gca(),'type','axes'),'fontSize',20,'fontname','Tahoma')                % Replace all axes text properties in the plot

load('Waveform');                                                               % Waveform aus Datei laden (Variable FilterTest)

%%%%% Variieren Sie die Fensterformen
window = hann(Datapoints);
% window = hamming(Datapoints);
% window = blackman(Datapoints);
% window = ones(Datapoints, 1);


MBI_setRunFlag('runflag');

while(runflag)

    [data_raw, data] = singleACQ(visa_obj, 104);                                % Get data from DAQ
    data = data_raw/2^15*10;                                                    % Calculate data in V, should be done in singleACQ() but does not work inR2013B!!!
 
    
    %%%%% Aufgabe 2: Bitte hier das digitale Filter realisieren.--------------  
    %%%%%mit filter()-----

%      set(ah12, 'XData', (0:(Datapoints-1))./Samplerate*1000, 'YData', .....);     % Update filtered signal in the plot

    %%%%%mit filtfilt()----

%     set(ah12, 'XData', (0:(Datapoints-1))./Samplerate*1000, 'YData', .....);     % Update filtered signal in the plot

    %%%%% Ende Aufgabe 2-------------------------------------------------------
    
    
    set(ah11, 'XData', (0:(Datapoints-1))./Samplerate*1000, 'YData', data);     % Update input signal in the plot
    
    data_windowed = data.*window;                                               % window input signal
    set(ah31, 'XData', (0:(Datapoints-1))./Samplerate*1000, 'YData', data);     % Update input signal in the plot
    set(ah32, 'XData', (0:(Datapoints-1))./Samplerate*1000, 'YData', data_windowed);     % Update input signal in the plot

    

    
    %%%%% Aufgabe 1: Bitte hier die FFT des gefensterten Signals realisieren.----
    %%%%% Verwenden Sie fft()
% 	NFFT = 2^nextpow2(length(data_windowed));         % Calculate the number of points for the fft to the next power of 2
%     Y = .....;        % Calculate the fft     
%     f = .....; 
% 
%     set(ah21, 'XData', f/1000, 'YData', .....[dB]);          % Update fft signal in the plot __
    %%%%% Ende Aufgabe 1-------------------------------------------------------


    
    
     fprintf(visa_obj, 'OUTP OFF');                                              % Output off
     write_ao(visa_obj, FilterTest, FilterTest);                                    % Write analog output data __
    fprintf(visa_obj, 'APPL:USER (@201)');                                      % Select arbitary Waveform
    fprintf(visa_obj, 'APPL:USER (@202)');                                      % Select arbitary Waveform
    fprintf(visa_obj, 'OUTP ON');                                               % Output on
    
    pause(0.1);
end


