% ****************************************************************
% ************ Messtechnik-Labor Bodediagramm-Skript *************
% ****************************************************************

% Institut f�r Automatisierungs- und Regelungstechnik (ACIN)
% Author: MBI
% Date: 21.10.2015
% Version: 1.03

%%  Initialise Connection
cd 'C:\Users\MT-LAB\Desktop\MT-LU5\Bodediagramm';

if (exist('osci', 'var'))                                                                       % If var osci exists
    fclose(osci);                                                                               % Close connection
end

close all; clear all; clc; 

%VISA_adress = '...........';

osci = visa('agilent', VISA_adress);                                                            % Create visa objekt for oscilloscope
fopen(osci);                                                                                    % Open connection to oscilloscope

fprintf(osci, '*IDN?' );                                                                        % Get ID from oscilloscope
fscanf(osci)                                                                                    % Read answer

%% Settings oscilloscope
clc;
error = 0;
error = set_command(osci, ':SYSTem:PRESet') + error;                                            % Reset oscilloscope equal to 'Default Setup' button at the scope

%***** Channel Settings *****

%%% Beide Kan�le haben die selben Einstellungen.
%%% Verwenden Sie osci_channel(.....)
% Keine Bandbreitenbegrenzung; DC Kopplung; Darstellung am Oszi; nicht invertiert;
% kein Offset; Tastkopfverh�ltnis beachten!; Spannungsbereich 22V
[ error ] = osci_init_channel( osci, 1, 0, 'DC', 1, 0, 0, 1, 1 );

%***** Trigger Settings *****
error = set_val_str(osci, ':TRIGger:MODE', 'EDGE') + error;                                     % Set Trigger Mode to edge

%%%%%  Einstellungen f�r den Trigger
%%% Schreiben Sie die Parameter direkt hinaus mit set_val_str(...) und
%%% set_val_num()
% DC Kopplung; Trigger Level 0V; Reject Filter off; steigende Flanke;
% Trigger Quelle: CH1

%%%%% Ende Einstellungen f�r den Trigger


%***** Timebase Settings *****
error = set_val_str(osci, ':TIMebase:MODE', 'MAIN') + error;                                    % Set Timebase Mode to standard
error = set_val_num(osci, ':TIMebase:POSition', 0) + error;                                     % Set Timebase Position to 0
error = set_val_num(osci, ':TIMebase:SCAle', 1e-3) + error;                                     % Set Timescale to 1ms/DIV

%***** Measurement Settings *****
error = set_command(osci, ':MEASure:CLEar') + error;                                            % Clear all Measurements
error = set_command(osci, ':MEASure:VRMS DISPlay, AC, CHANnel1') + error;                       % Add measurement VRMS full screen on channel 1
error = set_command(osci, ':MEASure:VRMS DISPlay, AC, CHANnel2') + error;                       % Add measurement VRMS full screen on channel 2
error = set_command(osci, ':MEASure:PHASe CHANnel1,CHANnel2') + error;                          % Add measurement Phase from channel1 to channel2

if (error)
    warning('Error occured while setting oscilloscope settings, please check!')
else
    sprintf('Scope initialised successfully!\n')
end

%% Settings wavegen
clc;
error = 0;

error = fgen_init_channel(osci, 'ONEM', 'SIN', 1e3, 5, 0) + error;                              % Init function generator of the scope

if (error)
    warning('Error occured while setting functiongenerator settings, please check!')
else
    sprintf('Functiongen initialised successfully!\n')
end

%% Measuring
close all; clear mag; clear phase; clc;

start_freq = 100e0;                                                                             % Start frequency
stop_freq = 30e3;                                                                               % Stop frequency
points = 50;                                                                                   % Number of points of the plot
sweep_type = 'lin';                                                                             % 'lin' or 'log'

error = set_val_num(osci, ':WGEN:OUTPut', 1);                                                   % Switch on wavegen Output
error = set_val_num(osci, ':WGEN:FREQuency', start_freq) + error;                               % Set wavegen output frequency
error = set_val_num(osci, ':TIMebase:RANGe', 3/start_freq) + error;                             % Set timebase to 3*f_start 

osci_check_range(osci, 1, 0);                                                                   % Check range of the scope on channel 1
osci_check_range(osci, 2, 0);                                                                   % Check range of the scope on channel 2

mag = zeros(points, 1);                                                                         % Create the magnitude vector
phase = zeros(points, 1);                                                                       % Create the phase vector

if (strcmp(sweep_type, 'lin'))
   freq = linspace(start_freq, stop_freq, points);                                              % If linear sweep the space frequency vector linear
else
   freq = logspace(log10(start_freq), log10(stop_freq), points);                                % Else space frequency vector logarithmic
end

%%%%% Was ist der Unterschied zwischen linspace und logspace (sehen Sie in der Hilfe nach). Was erscheint sinnvoller? 

freq = sort([freq 2245 2250 2255]);                                                             % Add some extra frequency points around the corner frequency
freq_size = size(freq);                                                                         % Get the size of the frequency vector


%%%%%% Generieren Sie die Messschleife:
%
% 1) i-te Messfrequenz am Wavegen einstellen
%    set_val_num(.....)
% 2) f�gen Sie folgende Zeilen ein (automatische Skalierung)
%
%     error = set_val_num(osci, ':TIMebase:RANGe', 3/freq(i)) + error;                            % Set Timescale to 3*T
%     osci_check_range(osci, 1, 1);                                                               % Check Range on channel 1
%     osci_check_range(osci, 2, 1);                                                               % Check Range on channel 2
%
% % 3) Auslesen der Messgr��en
% 
%     v_in_rms = get_val_num(.....);                       % Measure VRMS full screen on channel 1
%     v_out_rms = get_val_num(.....);                      % Measure VRMS full screen on channel 2
%     phase(i) = -get_val_num(osci, ':MEASure:PHASe? CHANnel1,CHANnel2');                         % Measure phase from channel1 to channel2
%     mag(i) = .......                                                      % Calculate magnitude
%     
%%%%%% Ende Messschleife

error = set_val_num(osci, ':WGEN:OUTPut', 0) + error;                                           % Switch off Output 1

if (error)
    warning('Error occured while measuring, please check!')
end

[b, a] = butter(2,5300,'s');                                                                    % Create 2nd order butterworth filter for comparison
[mag_filt4, phase_filt4, w_filt4] = bode(tf(b, a)*tf(b, a), logspace(0, 5, 10000));             % Build 4th order out of two 2nd order filters
[b, a] = butter(1,5300,'s');                                                                    % Create 1st order butterworth filter for comparison
[mag_filt1, phase_filt1, w_filt1] = bode(tf(b, a), logspace(0, 5, 10000));                      % Build 4th order out of two 2nd order filters

h1 = figure();                                                                                  % Create a figure with handle h1
set(h1, 'Name', 'Bodeplot');
maximize();                                                                                     % Show it maximised
subplot(2,1,1);                                                                                 % Create magniude subplot
semilogx(freq, mag, 'Linewidth', 2);                                                            % Plot measured magnitude
hold on;                                                                                        % Hold on for other graphs to be drawn
grid on;                                                                                        % Grid on
semilogx([1e1 5300 530000], [0 0 -40], 'm-.', 'Linewidth', 2);                                  % Asymptotic line 2nd order filter
semilogx([1e1 5300 530000], [0 0 -160], 'm-.', 'Linewidth', 2);                                 % Asymptotic line 4th order filter
semilogx(w_filt1, 20*log10(mag_filt1(:)), 'g--', 'Linewidth', 2);                               % Plot magnitude for the reference filter (model 1st order)
semilogx(w_filt4, 20*log10(mag_filt4(:)), 'g--', 'Linewidth', 2);                               % Plot magnitude for the reference filter (model 4th order)
ylabel('Amplitude (dB)', 'Fontsize', 20)                                                        % Set y label
axis([start_freq stop_freq -35 5]);                                                             % Select the axis

subplot(2,1,2);                                                                                 % Create phase subplot
semilogx(freq, unwrap(phase*pi/180)*180/pi, 'Linewidth', 2);                                    % Plot measured phase
hold on;                                                                                        % Hold on for other graphs to be drawn
grid on;                                                                                        % Grid on
plot(w_filt1, phase_filt1(:), 'g--', 'Linewidth', 2);                                           % Plot phase for the reference filter (model 1st order)
plot(w_filt4, phase_filt4(:), 'g--', 'Linewidth', 2);                                           % Plot phase for the reference filter (model 4th order)
xlabel('Frequency (Hz)', 'Fontsize', 20);                                                       % Set x label
ylabel('Phase (�)', 'Fontsize', 20);                                                            % Set y label
axis([start_freq stop_freq -360 10]);                                                           % Select the axis

set(findall(h1,'type','text'),'fontSize',12,'fontname','Tahoma')                                % Replace all text properties in the plot
set(findall(h1,'type','axes'),'fontSize',12,'fontname','Tahoma')                                % Replace all axes text properties in the plot